import QtQuick 1.1
import QtWidgetProxies 1.0
import "../common/widgets" as Widgets

Flickable
{
    id: tableViewItem
    property alias model: listView.model

    contentHeight: (model.length + 1) * rowHeight
    contentWidth: headerRow.width
    boundsBehavior: Flickable.StopAtBounds
    clip: true

    property list<SupportTableColumn> columns
    property bool headerVisible: false
    property Component headerRowDelegate
    property Component rowDelegate
    property int columnResizeGripWidth: 12
    property Component columnResizeIndicatorDelegate
    property Component columnResizeHeaderDelegate
    property Component tableFooter

    Component
    {
        id: tableRowComponent

        Item
        {
            id: tableRowItem
            width: name.width
            height: tableRow.height

            Row
            {
                id: tableRow
                width: parent.width
                z: 1
                Repeater
                {
                    model: columns
                    Item
                    {
                        width: modelData.width
                        height: cellLoader.height
                        Loader
                        {
                            id: cellLoader
                            sourceComponent: modelData.cellDelegate
                            width: parent.width
                            onLoaded:
                            {
                                if ("file" in item)
                                    item.file = getModelData();
                                if ("index" in item)
                                    item.index = getIndex();
                            }
                        }
                        Rectangle
                        {
                            id: rowDelimiter
                            color: "#dadada"
                            width: 1
                            height: cellLoader.height
                            x: modelData.width - width
                            visible: index === 0
                        }
                    }

                }
            }
            Loader
            {
                anchors.fill: tableRow
                sourceComponent: rowDelegate
                onLoaded: {
                    if ("index" in item)
                        item.index = getIndex();
                }
            }
            function getIndex() { return index; }
            function getModelData() { return modelData; }
        }
    }

    Component
    {
        id: emptyHeaderComponent
        Item
        {
            height: 1
        }
    }

    Component
    {
        id: headerCellComponent
        Item
        {
            id: headerCell
            width: modelData.width
            height: headerCellLoader.height

            MouseArea {
                id: sortMouseArea
                anchors.fill: parent
                anchors.leftMargin: columnResizeGripWidth / 2
                anchors.rightMargin: columnResizeGripWidth / 2
                hoverEnabled: true

                Widgets.Clickable {}

                onClicked:
                {
                    context.SortFiles(index)
                }
            }

            Loader
            {
                id: headerCellLoader
                sourceComponent: modelData.headerDelegate
                width: modelData.width
                onLoaded:
                {
                    item.text = columns[index].title
                }
            }

            Rectangle
            {
                id: headerDelimiter
                color: "#dadada"
                width: 1
                height: headerCellLoader.height
                x: modelData.width - width;
                visible: index === 0

                MouseArea
                {
                    id: mouseArea
                    x: -columnResizeGripWidth / 2
                    y: 0
                    anchors.left: parent.left
                    anchors.leftMargin: -2
                    anchors.right: parent.right
                    anchors.rightMargin: -2
                    height: parent.height
                    width: columnResizeGripWidth
                    drag.axis: Drag.XAxis
                    drag.minimumX: modelData.minimumWidth
                    drag.maximumX: name.width - headerView.model[1].minimumWidth
                    drag.target: headerDelimiter
                    onPositionChanged: modelData.width = headerDelimiter.x + headerDelimiter.width
                    hoverEnabled: true

                    onReleased:
                    {
                        Window.SetNormalCursor();
                    }

                    onEntered:
                    {
                        Window.SetResizableCursor();
                    }

                    onExited:
                    {
                        if(!pressed && !drag.active)
                            Window.SetNormalCursor();
                    }
                }
            }

            Loader
            {
                sourceComponent: columnResizeIndicatorDelegate
                x: modelData.width - width
                height: tableViewItem.height
                visible: index === 0
            }
        }
    }

    Column
    {
        id: columnItem
        width: parent.width

        Widgets.DPIScaler {changeHeight: true}

        Item
        {
            id: headerRowItem
            width: parent.width
            height: headerRow.height
            z: 2
            property int endPosX: -1
            property int startPosX: -1
            Row
            {
                id: headerRow
                z: 1
                visible: headerVisible
                Repeater
                {
                    id: headerView
                    model: columns
                    delegate: headerCellComponent
                }
            }
            Loader
            {
                anchors.fill: headerRow
                sourceComponent: headerVisible ? headerRowDelegate : undefined
            }
        }

        Item
        {
            id: name
            width: parent.width
            height: model.length * rowHeight

            Widgets.DPIScaler {changeHeight: true}

            ListView
            {
                id: listView
                boundsBehavior: Flickable.StopAtBounds
                interactive: false
                clip: true
                delegate: tableRowComponent
                anchors.fill: parent

                Widgets.DPIScaler {changeHeight: true}
            }
        }

        Column
        {
            width: parent.width
            height: (12 - model.length) * rowHeight

            Repeater
            {
                id: placeholders
                model: listView.count < 12 ? 12 - listView.count : 0
                delegate: rowDelegate

                Component.onCompleted:
                {
                    for (var i = 0, length = placeholders.count; i < length; i++) {
                        placeholders.itemAt(i).index = listView.count + i;
                    }
                }
            }
        }

        Loader
        {
            id: footerLoader
            width: parent.width
            sourceComponent: tableFooter
        }
    }
}
